@@ -20,6 +20,7 @@ import socket
 import sys
 import urllib.parse
 import urllib.request
+import warnings
 
 import numpy as np
 import pandas as pd
@@ -63,7 +64,7 @@ def confirmed(prompt=None, confirmation_required=True, resp=False):
 
     if confirmation_required:
         if prompt is None:
-            prompt_ = "Confirmed? "
+            prompt_ = "Confirmed?"
         else:
             prompt_ = copy.copy(prompt)
 
@@ -280,7 +281,7 @@ def get_number_of_chunks(file_or_obj, chunk_size_limit=50, binary=True):
     :type file_or_obj: str
     :param chunk_size_limit: the minimum limit of file size (in mebibyte i.e. MiB, or megabyte, i.e. MB)
         above which the function counts how many chunks there could be, defaults to ``50``;
-    :type chunk_size_limit: int
+    :type chunk_size_limit: int or float or None
     :param binary: whether to use binary (i.e. factorized by 1024) representation, defaults to ``True``;
         if ``binary=False``, use the decimal (or metric) representation (i.e. factorized by 10 ** 3)
     :type binary: bool
@@ -309,14 +310,52 @@ def get_number_of_chunks(file_or_obj, chunk_size_limit=50, binary=True):
 
     file_size_in_mb = round(size / (factor ** 2), 1)
 
-    if chunk_size_limit and file_size_in_mb > chunk_size_limit:
-        number_of_chunks = math.ceil(file_size_in_mb / chunk_size_limit)
+    if chunk_size_limit:
+        if file_size_in_mb > chunk_size_limit:
+            number_of_chunks = math.ceil(file_size_in_mb / chunk_size_limit)
+        else:
+            number_of_chunks = 1
     else:
         number_of_chunks = None
 
     return number_of_chunks
 
 
+def get_relative_path(pathname):
+    """
+    Get the relative or absolute path of ``pathname`` to the current working directory.
+
+    :param pathname: pathname (of a file or a directory)
+    :type pathname: str or os.PathLike[str]
+    :return: the relative or absolute path of ``path_to_file`` to the current working directory
+    :rtype: str or os.PathLike[str]
+
+    **Examples**::
+
+        >>> from pyhelpers.ops import get_relative_path
+        >>> import os
+
+        >>> rel_pathname = get_relative_path(pathname="")
+        >>> rel_pathname
+        ''
+        >>> rel_pathname = get_relative_path(pathname=os.path.join(os.getcwd(), "tests"))
+        >>> rel_pathname
+        'tests'
+
+        >>> # On Windows OS
+        >>> rel_pathname = get_relative_path(pathname="C:/Windows")
+        >>> rel_pathname
+        "C:/Windows"
+    """
+
+    try:
+        rel_path = os.path.relpath(pathname)
+    except ValueError:
+        rel_path = copy.copy(pathname)
+
+    return rel_path
+
+
 def find_executable(app_name, possibilities=None):
     """
     Get pathname of an executable file for a specified application.
@@ -943,6 +982,7 @@ def merge_dicts(*dicts):
         >>> dict_1 = merged_dict
         >>> dict_2 = {'b': 2, 'c': 4, 'd': [5, 6]}
         >>> merged_dict = merge_dicts(dict_1, dict_2)
+        >>> merged_dict
         {'a': 1, 'b': 2, 'c': [[3, 4], 4], 'd': [5, 6]}
     """
 
@@ -990,7 +1030,7 @@ def detect_nan_for_str_column(data_frame, column_names=None):
         Manchester   383819    398052
         Leeds        582044    152953
 
-        >>> dat.loc['Leeds', 'Northing'] = None
+        >>> dat.loc['Leeds', 'Latitude'] = None
         >>> dat
                     Easting  Northing
         City
@@ -1092,22 +1132,19 @@ def parse_csr_matrix(path_to_csr, verbose=False, **kwargs):
 
         >>> from pyhelpers.ops import parse_csr_matrix
         >>> from pyhelpers.dirs import cd
-        >>> import numpy
-        >>> import scipy.sparse
+        >>> from scipy.sparse import csr_matrix, save_npz
 
-        >>> data_ = numpy.array([1, 2, 3, 4, 5, 6])
-        >>> indices_ = numpy.array([0, 2, 2, 0, 1, 2])
-        >>> indptr_ = numpy.array([0, 2, 3, 6])
+        >>> data_ = [1, 2, 3, 4, 5, 6]
+        >>> indices_ = [0, 2, 2, 0, 1, 2]
+        >>> indptr_ = [0, 2, 3, 6]
 
-        >>> csr_m = scipy.sparse.csr_matrix((data_, indices_, indptr_), shape=(3, 3))
+        >>> csr_m = csr_matrix((data_, indices_, indptr_), shape=(3, 3))
         >>> csr_m
         <3x3 sparse matrix of type '<class 'numpy.int32'>'
             with 6 stored elements in Compressed Sparse Row format>
 
         >>> path_to_csr_npz = cd("tests\\data", "csr_mat.npz")
-        >>> numpy.savez_compressed(path_to_csr_npz, indptr=csr_m.indptr,
-        ...                        indices=csr_m.indices, data=csr_m.data,
-        ...                        shape=csr_m.shape)
+        >>> save_npz(path_to_csr_npz, csr_m)
 
         >>> parsed_csr_mat = parse_csr_matrix(path_to_csr_npz, verbose=True)
         Loading "\\tests\\data\\csr_mat.npz" ... Done.
@@ -1123,7 +1160,7 @@ def parse_csr_matrix(path_to_csr, verbose=False, **kwargs):
     scipy_sparse = _check_dependency(name='scipy.sparse')
 
     if verbose:
-        print("Loading \"\\{}\"".format(os.path.relpath(path_to_csr)), end=" ... ")
+        print("Loading \"\\{}\"".format(get_relative_path(path_to_csr)), end=" ... ")
 
     try:
         csr_loader = np.load(path_to_csr, **kwargs)
@@ -1163,24 +1200,24 @@ def swap_cols(array, c1, c2, as_list=False):
         >>> from pyhelpers.ops import swap_cols
         >>> from pyhelpers._cache import example_dataframe
 
-        >>> example_arr = example_dataframe().to_numpy(dtype=int)
+        >>> example_arr = example_dataframe(osgb36=True).to_numpy(dtype=int)
         >>> example_arr
-        array([[530034, 180381],
-               [406689, 286822],
-               [383819, 398052],
-               [582044, 152953]])
+        array([[530039, 180371],
+               [406705, 286868],
+               [383830, 398113],
+               [430147, 433553]])
 
         >>> # Swap the 0th and 1st columns
         >>> new_arr = swap_cols(example_arr, c1=0, c2=1)
         >>> new_arr
-        array([[180381, 530034],
-               [286822, 406689],
-               [398052, 383819],
-               [152953, 582044]])
+        array([[180371, 530039],
+               [286868, 406705],
+               [398113, 383830],
+               [433553, 430147]])
 
         >>> new_list = swap_cols(example_arr, c1=0, c2=1, as_list=True)
         >>> new_list
-        [[180381, 530034], [286822, 406689], [398052, 383819], [152953, 582044]]
+        [[180371, 530039], [286868, 406705], [398113, 383830], [433553, 430147]]
     """
 
     array_ = array.copy()
@@ -1212,24 +1249,24 @@ def swap_rows(array, r1, r2, as_list=False):
         >>> from pyhelpers.ops import swap_rows
         >>> from pyhelpers._cache import example_dataframe
 
-        >>> example_arr = example_dataframe().to_numpy(dtype=int)
+        >>> example_arr = example_dataframe(osgb36=True).to_numpy(dtype=int)
         >>> example_arr
-        array([[530034, 180381],
-               [406689, 286822],
-               [383819, 398052],
-               [582044, 152953]])
+        array([[406705, 286868],
+               [530039, 180371],
+               [383830, 398113],
+               [430147, 433553]])
 
         >>> # Swap the 0th and 1st rows
         >>> new_arr = swap_rows(example_arr, r1=0, r2=1)
         >>> new_arr
-        array([[406689, 286822],
-               [530034, 180381],
-               [383819, 398052],
-               [582044, 152953]])
+        array([[406705, 286868],
+               [530039, 180371],
+               [383830, 398113],
+               [430147, 433553]])
 
         >>> new_list = swap_rows(example_arr, r1=0, r2=1, as_list=True)
         >>> new_list
-        [[406689, 286822], [530034, 180381], [383819, 398052], [582044, 152953]]
+        [[406705, 286868], [530039, 180371], [383830, 398113], [430147, 433553]]
     """
 
     array_ = array.copy()
@@ -1261,24 +1298,24 @@ def np_shift(array, step, fill_value=np.nan):
         >>> from pyhelpers.ops import np_shift
         >>> from pyhelpers._cache import example_dataframe
 
-        >>> arr = example_dataframe().to_numpy()
+        >>> arr = example_dataframe(osgb36=True).to_numpy()
         >>> arr
-        array([[530034, 180381],
-               [406689, 286822],
-               [383819, 398052],
-               [582044, 152953]], dtype=int64)
+        array([[530039.5588445, 180371.6801655],
+               [406705.8870136, 286868.1666422],
+               [383830.0390357, 398113.0558309],
+               [430147.4473539, 433553.3271173]])
 
         >>> np_shift(arr, step=-1)
-        array([[406689., 286822.],
-               [383819., 398052.],
-               [582044., 152953.],
-               [    nan,     nan]])
+        array([[406705.8870136, 286868.1666422],
+               [383830.0390357, 398113.0558309],
+               [430147.4473539, 433553.3271173],
+               [           nan,            nan]])
 
         >>> np_shift(arr, step=1, fill_value=0)
         array([[     0,      0],
-               [530034, 180381],
-               [406689, 286822],
-               [383819, 398052]], dtype=int64)
+               [530039, 180371],
+               [406705, 286868],
+               [383830, 398113]])
     """
 
     result = np.empty_like(array, dtype=type(fill_value))  # np.zeros_like(array)
@@ -1500,9 +1537,21 @@ def cmap_discretisation(cmap, n_colours):
         >>> plt.close()
     """
 
-    matplotlib_cm, matplotlib_colors = map(_check_dependency, ['matplotlib.cm', 'matplotlib.colors'])
+    try:  # new version
+        mpl_cm, mpl_colors = map(_check_dependency, ['matplotlib.colormaps', 'matplotlib.colors'])
+    except ModuleNotFoundError:
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore')
+            mpl_cm, mpl_colors = map(_check_dependency, ['matplotlib.cm', 'matplotlib.colors'])
 
-    cmap_ = matplotlib_cm.get_cmap(cmap) if isinstance(cmap, str) else copy.copy(cmap)
+    if isinstance(cmap, str):
+        try:  # new version
+            cmap_ = mpl_cm[cmap]
+        except TypeError:
+            cmap_ = mpl_cm.get_cmap(cmap)
+    else:
+        assert isinstance(cmap, mpl_colors.ListedColormap)
+        cmap_ = copy.copy(cmap)
 
     colours_i = np.concatenate((np.linspace(0, 1., n_colours), (0., 0., 0., 0.)))
     colours_rgba = cmap_(colours_i)
@@ -1514,7 +1563,7 @@ def cmap_discretisation(cmap, n_colours):
             (indices[x], colours_rgba[x - 1, ki], colours_rgba[x, ki]) for x in range(n_colours + 1)
         ]
 
-    colour_map = matplotlib_colors.LinearSegmentedColormap(cmap.name + '_%d' % n_colours, c_dict, 1024)
+    colour_map = mpl_colors.LinearSegmentedColormap(cmap.name + '_%d' % n_colours, c_dict, 1024)
 
     return colour_map
 
@@ -1547,9 +1596,12 @@ def colour_bar_index(cmap, n_colours, labels=None, **kwargs):
     **Examples**::
 
         >>> from pyhelpers.ops import colour_bar_index
+        >>> import matplotlib
         >>> import matplotlib.pyplot as plt
         >>> import matplotlib.cm
 
+        >>> matplotlib.use('TkAgg')
+
         >>> plt.figure(figsize=(2, 6))
 
         >>> cbar = colour_bar_index(cmap=matplotlib.cm.get_cmap('Accent'), n_colours=5)
@@ -1602,15 +1654,21 @@ def colour_bar_index(cmap, n_colours, labels=None, **kwargs):
         >>> plt.close(fig='all')
     """
 
-    matplotlib_cm, matplotlib_pyplot = map(_check_dependency, ['matplotlib.cm', 'matplotlib.pyplot'])
+    try:  # new version
+        mpl_cm, mpl_plt = map(_check_dependency, ['matplotlib.colormaps', 'matplotlib.pyplot'])
+    except ModuleNotFoundError:
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore')
+            mpl_cm, mpl_plt = map(_check_dependency, ['matplotlib.cm', 'matplotlib.pyplot'])
 
-    cmap = cmap_discretisation(cmap, n_colours)
+    # assert isinstance(cmap, mpl_cm.ListedColormap)
+    cmap_ = cmap_discretisation(cmap, n_colours)
 
-    mappable = matplotlib_cm.ScalarMappable(cmap=cmap)
+    mappable = mpl_cm.ScalarMappable(cmap=cmap_)
     mappable.set_array(np.array([]))
     mappable.set_clim(-0.5, n_colours + 0.5)
 
-    colour_bar = matplotlib_pyplot.colorbar(mappable, **kwargs)
+    colour_bar = mpl_plt.colorbar(mappable=mappable, ax=mpl_plt.gca(), **kwargs)
     colour_bar.set_ticks(np.linspace(0, n_colours, n_colours))
     colour_bar.set_ticklabels(range(n_colours))
 
@@ -1835,11 +1893,13 @@ def init_requests_session(url, max_retries=5, backoff_factor=0.1, retry_status='
 
 
 class _FakeUserAgentParser(html.parser.HTMLParser):
-    def __init__(self):
+
+    def __init__(self, browser_name):
         super().__init__()
         self.reset()
         self.recording = 0
         self.data = []
+        self.browser_name = browser_name
 
     def error(self, message):
         pass
@@ -1854,7 +1914,7 @@ class _FakeUserAgentParser(html.parser.HTMLParser):
 
         if tag == 'a':
             for name, link in attrs:
-                if name == 'href' and link.startswith('/index.php?id='):
+                if name == 'href' and link.startswith(f'/{self.browser_name}') and link.endswith('.php'):
                     break
                 else:
                     return
@@ -1896,12 +1956,14 @@ def _user_agent_strings(browser_names=None, dump_dat=True):
 
     user_agent_strings = {}
     for browser_name in browser_names_:
-        response = requests.get(url=resource_url + f'?name={browser_name.replace(" ", "+")}')
-        fua_parser = _FakeUserAgentParser()
+        # url = resource_url.replace('useragentstring.php', browser_name.replace(" ", "+") + '/')
+        url = resource_url + f'?name={browser_name.replace(" ", "+")}'
+        response = requests.get(url=url)
+        fua_parser = _FakeUserAgentParser(browser_name=browser_name)
         fua_parser.feed(response.text)
         user_agent_strings[browser_name] = list(set(fua_parser.data))
 
-    if dump_dat:
+    if dump_dat and all(user_agent_strings.values()):
         path_to_json = pkg_resources.resource_filename(__name__, "data\\user-agent-strings.json")
 
         json_out = open(path_to_json, mode='w')
@@ -2112,7 +2174,7 @@ def _download_file_from_url(response, path_to_file):
     total_iter = file_size // chunk_size
 
     progress = tqdm_.tqdm(
-        desc=f'"{os.path.relpath(path_to_file)}"', total=total_iter, unit='B', unit_scale=True,
+        desc=f'"{get_relative_path(path_to_file)}"', total=total_iter, unit='B', unit_scale=True,
         unit_divisor=unit_divisor)
 
     contents = response.iter_content(chunk_size=chunk_size, decode_unicode=True)
